import React, { useCallback, useEffect, useRef, useState } from 'react'
import './totalAnalysisChart.scss'
import * as d3 from 'd3'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { formatNumberValues } from 'utils/utils'
import { DateTime } from 'luxon'
import { convertDateToMonthString } from 'utils/date'
import EstimatedConsumptionModal from 'components/ConsumptionVisualizer/EstimatedConsumptionModal'
import { DataloadValueDetail } from 'models'

interface PieProps {
  innerRadius: number
  outerRadius: number
  dataloadValueDetailArray: DataloadValueDetail[]
  width: number
  height: number
  totalValue: number
  currentAnalysisDate: DateTime
}

const PieChart: React.FC<PieProps> = ({
  innerRadius,
  outerRadius,
  dataloadValueDetailArray,
  width,
  height,
  totalValue,
  currentAnalysisDate,
}: PieProps) => {
  const ref = useRef(null)
  const { t } = useI18n()
  const createPie = d3.pie().sort(null)
  const arcWidth = outerRadius - innerRadius
  const [openEstimationModal, setOpenEstimationModal] = useState<boolean>(false)
  const toggleEstimationModal = useCallback(() => {
    setOpenEstimationModal(prev => !prev)
  }, [])

  useEffect(() => {
    const dataloadArray: number[] = dataloadValueDetailArray.map(
      dataload => dataload.value
    )
    const data = createPie(dataloadArray)
    const group = d3.select(ref.current)
    const groupWithData = group.selectAll('g.arc').data(data)
    const colors = ['#D87B39', '#3A98EC', '#45D1B8']

    const createArc: any = d3
      .arc()
      .innerRadius(innerRadius)
      .outerRadius(outerRadius)
    groupWithData.exit().remove()

    const groupWithUpdate = groupWithData
      .enter()
      .append('g')
      .attr('class', 'arc')
      .attr('filter', 'url(#glow)')

    const path = groupWithUpdate
      .append('path')
      .merge(groupWithData.select('path.arc'))

    path
      .attr('class', 'arc')
      .attr('d', createArc)
      .attr('fill', (_d, i) => colors[i])
  }, [createPie, dataloadValueDetailArray, innerRadius, outerRadius])

  return (
    <div
      className="pie-container"
      style={{
        width: width,
        height: height,
      }}
    >
      <svg width={width} height={height}>
        <defs>
          <filter id="glow" height="300%" width="300%" x="-75%" y="-75%">
            <feGaussianBlur stdDeviation="10" result="coloredBlur" />
            <feMerge>
              <feMergeNode in="coloredBlur" />
              <feMergeNode in="SourceGraphic" />
            </feMerge>
          </filter>
        </defs>
        <g ref={ref} transform={`translate(${outerRadius} ${outerRadius})`} />
      </svg>
      <div
        className="pie-center"
        style={{
          width: width - arcWidth,
          height: height - arcWidth,
          top: arcWidth / 2,
          left: arcWidth - arcWidth / 2,
        }}
      >
        <div className="text-36-bold">
          {formatNumberValues(totalValue)}
          <span className="euro-unit">{`${t('FLUID.MULTIFLUID.UNIT')}`}</span>
        </div>
        <div className="text-16-normal date">
          {t('analysis_pie.month') +
            convertDateToMonthString(currentAnalysisDate).substring(3)}
        </div>
        <div
          className="text-14-normal estimation-text"
          onClick={toggleEstimationModal}
        >
          <span className="estimated">{t('analysis_pie.estimation')}</span>
          <span className="estimated">{t('analysis_pie.estimation2')}</span>
        </div>
        <div
          className="circle"
          style={{
            width: width - arcWidth * 2,
            height: height - arcWidth * 2,
            top: arcWidth / 2,
            left: arcWidth - arcWidth / 2,
          }}
        ></div>
      </div>
      <EstimatedConsumptionModal
        open={openEstimationModal}
        handleCloseClick={toggleEstimationModal}
      />
    </div>
  )
}

export default PieChart
