import React, { useCallback, useEffect, useState } from 'react'
import './duelUnlocked.scss'
import { Client, useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { useDispatch } from 'react-redux'
import { UserChallenge } from 'models'
import { setChallengeConsumption } from 'store/challenge/challenge.actions'
import ChallengeService from 'services/challenge.service'
import { formatNumberValues, importIconbyId } from 'utils/utils'

import defaultDuelIcon from 'assets/icons/visu/challenge/CHALLENGE0001.svg'
import defaultIcon from 'assets/icons/visu/duel/default.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Button from '@material-ui/core/Button'
import { UserChallengeUpdateFlag } from 'enum/userChallenge.enum'
import { UsageEventType } from 'enum/usageEvent.enum'
import UsageEventService from 'services/usageEvent.service'

interface DuelUnlockedProps {
  userChallenge: UserChallenge
}

const DuelUnlocked: React.FC<DuelUnlockedProps> = ({
  userChallenge,
}: DuelUnlockedProps) => {
  const client: Client = useClient()
  const dispatch = useDispatch()
  const { t } = useI18n()

  const [duelIcon, setDuelIcon] = useState(defaultIcon)
  const average: string = formatNumberValues(
    userChallenge.duel.threshold
  ).toString()
  const duration: string = userChallenge.duel.duration.days.toString()
  const description: string = userChallenge.duel.description
    .replace('#CONSUMPTION', average)
    .replace('#DURATION', duration)

  const launchDuel = useCallback(async () => {
    const challengeService = new ChallengeService(client)
    const updatedChallenge = await challengeService.updateUserChallenge(
      userChallenge,
      UserChallengeUpdateFlag.DUEL_START
    )
    const dataloads = await challengeService.getUserChallengeDataload(
      updatedChallenge
    )
    await UsageEventService.addEvent(client, {
      type: UsageEventType.DUEL_LAUNCH_EVENT,
      target: userChallenge.duel.id,
      context: userChallenge.id,
    })
    dispatch(setChallengeConsumption(updatedChallenge, dataloads))
  }, [client, dispatch, userChallenge])

  useEffect(() => {
    async function handleEcogestureIcon() {
      const icon = await importIconbyId(userChallenge.id, 'challenge')
      if (icon) {
        setDuelIcon(icon)
      } else {
        setDuelIcon(defaultDuelIcon)
      }
    }
    handleEcogestureIcon()
  }, [userChallenge])

  return (
    <>
      <div className="duel-unlocked-container">
        <StyledIcon className="duel-icon" icon={duelIcon} size={219} />
        <div className="duel-description text-20-italic">{`"${description}"`}</div>
        <div className="duel-title text-16-normal">
          {userChallenge.duel.title}
        </div>
        <div className="duel-average-info text-18-normal">
          {`${t('duel.average_info', {
            average,
            // eslint-disable-next-line camelcase
            smart_count: average,
          })}`}
        </div>
        <div className="button-start">
          <Button
            aria-label={t('duel.accessibility.button_start_duel')}
            onClick={launchDuel}
            classes={{
              root: 'btn-secondary-negative',
              label: 'text-16-normal',
            }}
          >
            {t('duel.button_start')}
          </Button>
        </div>
      </div>
    </>
  )
}

export default DuelUnlocked
