import { Tab, Tabs } from '@material-ui/core'
import classNames from 'classnames'
import Content from 'components/Content/Content'
import EcogestureList from 'components/Ecogesture/EcogestureList'
import CozyBar from 'components/Header/CozyBar'
import Header from 'components/Header/Header'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { EcogestureStatus } from 'enum/ecogesture.enum'
import { Ecogesture } from 'models'
import { ProfileEcogesture } from 'models/profileEcogesture.model'
import React, { useCallback, useEffect, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import { useHistory, useLocation } from 'react-router-dom'
import EcogestureService from 'services/ecogesture.service'
import { AppStore } from 'store'
import { updateProfile } from 'store/profile/profile.actions'
import { getSeason } from 'utils/utils'
import EcogestureEmptyList from './EcogestureEmptyList'
import EcogestureInitModal from './EcogestureInitModal'
import EcogestureReinitModal from './EcogestureReinitModal'
import './ecogestureView.scss'

interface TabPanelProps {
  children?: React.ReactNode
  index: number
  value: number
}
const TabPanel: React.FC<TabPanelProps> = ({
  children,
  index,
  value,
}: TabPanelProps) => {
  return (
    <div
      role="tabpanel"
      hidden={value !== index}
      id={`simple-tabpanel-${index}`}
      aria-labelledby={`simple-tab-${index}`}
    >
      {children}
    </div>
  )
}

const EcogestureView: React.FC = () => {
  const [headerHeight, setHeaderHeight] = useState<number>(0)
  const defineHeaderHeight = (height: number) => {
    setHeaderHeight(height)
  }
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useDispatch()
  const tab = new URLSearchParams(useLocation().search).get('tab')

  const profileType = useSelector((state: AppStore) => state.ecolyo.profileType)
  const profileEcogesture: ProfileEcogesture = useSelector(
    (state: AppStore) => state.ecolyo.profileEcogesture
  )

  const { haveSeenEcogestureModal, isProfileEcogestureCompleted } = useSelector(
    (state: AppStore) => state.ecolyo.profile
  )
  const [tabValue, setTabValue] = useState<EcogestureStatus>(
    tab ? parseInt(tab) : EcogestureStatus.OBJECTIVE
  )
  const history = useHistory()
  const [isLoaded, setIsLoaded] = useState<boolean>(false)
  const [allEcogestureList, setAllEcogestureList] = useState<Ecogesture[]>([])
  const [doingEcogestureList, setDoingEcogestureList] = useState<Ecogesture[]>(
    []
  )
  const [objectiveEcogestureList, setObjectiveEcogestureList] = useState<
    Ecogesture[]
  >([])
  const [totalViewed, setTotalViewed] = useState<number>(0)
  const [totalAvailable, setTotalAvailable] = useState<number>(0)
  const [openEcogestureInitModal, setOpenEcogestureInitModal] =
    useState<boolean>(!haveSeenEcogestureModal)
  const [openEcogestureReinitModal, setOpenEcogestureReinitModal] =
    useState<boolean>(false)

  const handleReinitClick = useCallback(() => {
    setOpenEcogestureReinitModal(true)
  }, [])

  const handleLaunchForm = useCallback(async () => {
    dispatch(updateProfile({ haveSeenEcogestureModal: true }))
    setOpenEcogestureInitModal(false)
    history.push('/ecogesture-form?modal=false')
  }, [dispatch, history])

  const handleCloseEcogestureInitModal = useCallback(async () => {
    dispatch(updateProfile({ haveSeenEcogestureModal: true }))
    setOpenEcogestureInitModal(false)
  }, [dispatch])

  const handleLaunchReinit = useCallback(async () => {
    setOpenEcogestureReinitModal(false)
    setIsLoaded(false)
    const ecogestureService = new EcogestureService(client)
    const reset = await ecogestureService.reinitAllEcogestures()
    if (reset) {
      setOpenEcogestureReinitModal(false)
      setIsLoaded(true)
      history.push('/ecogesture-form?modal=true')
    }
  }, [client, history])

  const handleCloseEcogestureReinitModal = useCallback(() => {
    setOpenEcogestureReinitModal(false)
  }, [])

  const handleChange = useCallback(
    (event: React.ChangeEvent<{}>, newValue: any) => {
      event.preventDefault()
      const params = new URLSearchParams()
      params.append('tab', newValue.toString())
      history.push({ search: params.toString() })
      setTabValue(newValue)
    },
    [history]
  )

  const tabProps = useCallback((index: number) => {
    return {
      id: `simple-tab-${index}`,
      'aria-controls': `simple-tabpanel-${index}`,
    }
  }, [])

  const getLabel = useCallback(
    (_tab: number) => {
      return (
        <>
          {t(`ecogesture.title_tab_${_tab}`)}
          <br></br>
          {_tab === EcogestureStatus.OBJECTIVE &&
            `(${objectiveEcogestureList.length})`}
          {_tab === EcogestureStatus.DOING && `(${doingEcogestureList.length})`}
          {_tab === EcogestureStatus.ALL && `(${allEcogestureList.length})`}
        </>
      )
    },
    [
      allEcogestureList.length,
      doingEcogestureList.length,
      objectiveEcogestureList.length,
      t,
    ]
  )

  useEffect(() => {
    let subscribed = true
    async function loadEcogestures() {
      const ecogestureService = new EcogestureService(client)
      const dataAll = await ecogestureService.getAllEcogestures(getSeason())
      const availableList: Ecogesture[] =
        await ecogestureService.getEcogestureListByProfile(profileEcogesture)
      const filteredList: Ecogesture[] = availableList.filter(
        (ecogesture: Ecogesture) => ecogesture.viewedInSelection === false
      )
      if (subscribed && dataAll) {
        const doing = dataAll.filter(ecogesture => ecogesture.doing === true)
        const objective = dataAll.filter(
          ecogesture => ecogesture.objective === true
        )
        setAllEcogestureList(dataAll)
        setObjectiveEcogestureList(objective)
        setDoingEcogestureList(doing)
        setTotalAvailable(availableList.length)
        setTotalViewed(availableList.length - filteredList.length)
      }
      setIsLoaded(true)
    }
    loadEcogestures()
    return () => {
      subscribed = false
    }
  }, [client, profileEcogesture, profileType])

  return (
    <>
      <CozyBar titleKey={'common.title_ecogestures'} />
      {!isLoaded && (
        <Content height={headerHeight}>
          <div className="ecogesture-spinner" aria-busy="true">
            <Loader />
          </div>
        </Content>
      )}
      {isLoaded && (
        <>
          <Header
            setHeaderHeight={defineHeaderHeight}
            desktopTitleKey={'common.title_ecogestures'}
          >
            <Tabs
              value={tabValue}
              className="ecogestures-tabs"
              aria-label="ecogestures-tabs"
              onChange={handleChange}
              TabIndicatorProps={{ className: 'indicator-tab' }}
              centered={true}
            >
              <Tab
                label={getLabel(EcogestureStatus.OBJECTIVE)}
                className={classNames('single-tab', {
                  ['active']: tabValue === EcogestureStatus.OBJECTIVE,
                })}
                {...tabProps(EcogestureStatus.OBJECTIVE)}
              />
              <Tab
                label={getLabel(EcogestureStatus.DOING)}
                className={classNames('single-tab', {
                  ['active']: tabValue === EcogestureStatus.DOING,
                })}
                {...tabProps(EcogestureStatus.DOING)}
              />
              <Tab
                label={getLabel(EcogestureStatus.ALL)}
                className={classNames('single-tab', {
                  ['active']: tabValue === EcogestureStatus.ALL,
                })}
                {...tabProps(EcogestureStatus.ALL)}
              />
            </Tabs>
          </Header>

          <Content height={headerHeight}>
            <TabPanel value={tabValue} index={EcogestureStatus.OBJECTIVE}>
              {isProfileEcogestureCompleted &&
                (totalAvailable === totalViewed &&
                objectiveEcogestureList.length === 0 ? (
                  <EcogestureEmptyList
                    setTab={setTabValue}
                    isObjective={true}
                    isSelectionDone={true}
                    handleReinitClick={handleReinitClick}
                  />
                ) : (
                  <EcogestureList
                    list={objectiveEcogestureList}
                    displaySelection={totalAvailable !== totalViewed}
                    selectionTotal={totalAvailable}
                    selectionViewed={totalViewed}
                    index={EcogestureStatus.OBJECTIVE}
                    handleReinitClick={handleReinitClick}
                  />
                ))}
              {!isProfileEcogestureCompleted && (
                <EcogestureEmptyList
                  setTab={setTabValue}
                  isObjective={true}
                  isSelectionDone={false}
                  handleReinitClick={handleReinitClick}
                />
              )}
            </TabPanel>

            <TabPanel value={tabValue} index={EcogestureStatus.DOING}>
              {isProfileEcogestureCompleted &&
                (totalAvailable === totalViewed &&
                doingEcogestureList.length === 0 ? (
                  <EcogestureEmptyList
                    setTab={setTabValue}
                    isObjective={false}
                    isSelectionDone={true}
                    handleReinitClick={handleReinitClick}
                  />
                ) : (
                  <EcogestureList
                    list={doingEcogestureList}
                    displaySelection={totalAvailable !== totalViewed}
                    selectionTotal={totalAvailable}
                    selectionViewed={totalViewed}
                    index={EcogestureStatus.DOING}
                    handleReinitClick={handleReinitClick}
                  />
                ))}
              {!isProfileEcogestureCompleted && (
                <EcogestureEmptyList
                  setTab={setTabValue}
                  isObjective={false}
                  isSelectionDone={false}
                  handleReinitClick={handleReinitClick}
                />
              )}
            </TabPanel>

            <TabPanel value={tabValue} index={EcogestureStatus.ALL}>
              {allEcogestureList.length && (
                <EcogestureList
                  list={allEcogestureList}
                  displaySelection={false}
                  selectionTotal={totalAvailable}
                  selectionViewed={totalViewed}
                  index={EcogestureStatus.ALL}
                  handleReinitClick={handleReinitClick}
                />
              )}
            </TabPanel>
          </Content>
        </>
      )}
      {openEcogestureInitModal && (
        <EcogestureInitModal
          open={openEcogestureInitModal}
          handleCloseClick={handleCloseEcogestureInitModal}
          handleLaunchForm={handleLaunchForm}
        />
      )}
      {openEcogestureReinitModal && (
        <EcogestureReinitModal
          open={openEcogestureReinitModal}
          handleCloseClick={handleCloseEcogestureReinitModal}
          handleLaunchReinit={handleLaunchReinit}
        />
      )}
    </>
  )
}

export default EcogestureView
