import { Button, IconButton } from '@material-ui/core'
import Dialog from '@material-ui/core/Dialog'
import CloseIcon from 'assets/icons/ico/close.svg'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import * as FileSaver from 'file-saver'
import { Datachart, TimePeriod } from 'models'
import React, { useCallback, useEffect } from 'react'
import ConsumptionDataManager from 'services/consumption.service'
import * as XLSX from 'xlsx'
import './exportLoadingModal.scss'

interface ExportLoadingModalProps {
  open: boolean
  handleCloseClick: () => void
  handleDone: (e?: any) => void
  selectedFluids: FluidType[]
}

const ExportLoadingModal: React.FC<ExportLoadingModalProps> = ({
  open,
  handleCloseClick,
  handleDone,
  selectedFluids,
}: ExportLoadingModalProps) => {
  const { t } = useI18n()
  const client = useClient()

  const fileType =
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=UTF-8'
  const fileExtension = '.xlsx'

  const exportToXlsx = (exportDataSheets: any, fileName: string) => {
    const wb = XLSX.utils.book_new()
    for (const dataSheet of exportDataSheets) {
      const ws = XLSX.utils.json_to_sheet(dataSheet.data)
      XLSX.utils.book_append_sheet(wb, ws, dataSheet.fluidName)
    }
    const excelBuffer = XLSX.write(wb, { bookType: 'xlsx', type: 'array' })
    const data = new Blob([excelBuffer], { type: fileType })
    FileSaver.saveAs(data, fileName + fileExtension)
  }

  const getExportDataFluid = useCallback(
    async (fluidType: FluidType): Promise<any> => {
      const consumptionService = new ConsumptionDataManager(client)
      const firstDataDate = await consumptionService.fetchAllFirstDateData(
        [fluidType],
        TimeStep.MONTH
      )
      const lastDataDate = await consumptionService.fetchAllLastDateData(
        [fluidType],
        TimeStep.MONTH
      )
      if (!firstDataDate[0] || !lastDataDate[0]) return null

      const timePeriod: TimePeriod = {
        startDate: firstDataDate[0],
        endDate: lastDataDate[0],
      }

      const dataLoad: Datachart | null = await consumptionService.getGraphData(
        timePeriod,
        TimeStep.MONTH,
        [fluidType],
        undefined,
        undefined,
        false,
        true
      )

      if (dataLoad?.actualData) {
        const exportDataFluid: any = {}
        exportDataFluid.fluidName = t(
          'FLUID.' + FluidType[fluidType] + '.LABEL'
        )
        exportDataFluid.data = []

        for (const data of dataLoad.actualData) {
          if (data.value != -1) {
            const dataRow: any = {}
            dataRow[t('export.month')] = data.date.month
              .toString()
              .padStart(2, '0')
            dataRow[t('export.year')] = data.date.year
            dataRow[
              `${t('export.consumption')} (${t(
                'FLUID.' + FluidType[fluidType] + '.UNIT'
              )})`
            ] = data.value
            exportDataFluid.data.push(dataRow)
          }
        }
        return exportDataFluid
      }
      return null
    },
    [client, t]
  )

  useEffect(() => {
    let subscribed = true
    const date = new Date()

    async function exportData(): Promise<void> {
      try {
        const exportDataSheets: any[] = []
        for (const fluidType of selectedFluids) {
          const exportDataFluid = await getExportDataFluid(fluidType)
          if (exportDataFluid) exportDataSheets.push(exportDataFluid)
        }
        await new Promise(r => setTimeout(r, 2000))
        if (subscribed) {
          exportToXlsx(
            exportDataSheets,
            'ecolyo_data_' + date.toLocaleDateString()
          )
          handleDone()
        }
      } catch (e) {
        handleDone(e)
      }
    }

    if (subscribed) {
      exportData()
    }
    return () => {
      subscribed = false
    }
  }, [client, getExportDataFluid, handleDone, selectedFluids, t])

  return (
    <Dialog
      open={open}
      onClose={(_event, reason) => {
        if (reason !== 'backdropClick' && reason !== 'escapeKeyDown') {
          handleCloseClick()
        }
      }}
      aria-labelledby={'accessibility-title'}
      classes={{
        root: 'modal-root',
        paper: 'modal-paper',
      }}
    >
      <div id={'accessibility-title'}>
        {t('export.modal_loading.accessibility_title')}
      </div>

      <div className="modal-loading-root">
        <div onClick={handleCloseClick}>
          <IconButton
            aria-label={t('export.button_close')}
            className="modal-paper-close-button"
            onClick={handleCloseClick}
          >
            <Icon icon={CloseIcon} size={18} />
          </IconButton>
        </div>
        <div className="content">
          <div className="icon-main">
            <Loader color="gold" />
          </div>
          <div className="text-16-bold text text1">
            {t('export.modal_loading.text1')}
          </div>
          <div className="text-16-normal text">
            {t('export.modal_loading.text2')}
          </div>
          <Button
            aria-label={t('export.modal_loading.button_cancel')}
            onClick={() => {
              handleCloseClick()
            }}
            classes={{
              root: 'btn-secondary-negative',
              label: 'text-16-bold',
            }}
            type="submit"
          >
            {t('export.modal_loading.button_cancel')}
          </Button>
        </div>
      </div>
    </Dialog>
  )
}

export default ExportLoadingModal
