import Button from '@material-ui/core/Button'
import { useClient } from 'cozy-client'
import {
  ERROR_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/flowEvents'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType } from 'enum/fluid.enum'
import { KonnectorError } from 'enum/konnectorError.enum'
import { Account } from 'models'
import React, { useCallback } from 'react'
import { useHistory } from 'react-router-dom'
import AccountService from 'services/account.service'
import './konnectorModal.scss'

interface KonnectorModalFooterProps {
  state: string | null
  error: KonnectorError | null
  fluidType: FluidType
  handleCloseClick: (isSuccess?: boolean) => void
  handleAccountDeletion: () => Promise<void>
  account: Account | null
  isUpdating: boolean
}

const KonnectorModalFooter: React.FC<KonnectorModalFooterProps> = ({
  state,
  error,
  fluidType,
  handleCloseClick,
  handleAccountDeletion,
  account,
  isUpdating,
}: KonnectorModalFooterProps) => {
  const { t } = useI18n()
  const client = useClient()
  const history = useHistory()
  const handleSGELoginRetry = useCallback(() => {
    handleCloseClick(state === SUCCESS_EVENT)
    history.push('/sge-connect')
  }, [handleCloseClick, history, state])

  const handleResetSGEAccount = useCallback(async () => {
    if (account) {
      const accountService = new AccountService(client)
      await accountService.deleteAccount(account)
      await handleAccountDeletion()
      history.push('/sge-connect')
    }
  }, [account, client, handleAccountDeletion, history])

  const errorButtons = () => {
    switch (error) {
      case KonnectorError.USER_ACTION_NEEDED:
        //INSEE CODE ERROR ENEDIS
        return (
          <Button
            aria-label={t('konnector_modal.accessibility.button_close')}
            onClick={() => handleCloseClick(state === SUCCESS_EVENT)}
            classes={{
              root: 'btn-highlight',
              label: 'text-16-bold',
            }}
            style={{ height: '40px' }}
          >
            <div>{t('konnector_modal.button_understood')}</div>
          </Button>
        )
      case KonnectorError.LOGIN_FAILED:
        //MISMATCH NAME / ADDRESS / PDL
        return (
          <Button
            aria-label={t('konnector_modal.accessibility.button_close')}
            onClick={() => handleCloseClick(state === SUCCESS_EVENT)}
            classes={{
              root: 'btn-highlight',
              label: 'text-16-bold',
            }}
            style={{ height: '40px' }}
          >
            <div>{t('konnector_modal.button_come_back_later')}</div>
          </Button>
        )
      case KonnectorError.TERMS_VERSION_MISMATCH:
        return (
          <div className="buttons">
            <Button
              aria-label={t('konnector_modal.accessibility.button_close')}
              onClick={() => handleCloseClick(state === SUCCESS_EVENT)}
              classes={{
                root: 'btn-secondary-negative',
                label: 'text-16-bold',
              }}
              style={{ height: '40px' }}
            >
              <div>{'Plus tard'}</div>
            </Button>
            <Button
              aria-label={t('konnector_modal.accessibility.button_close')}
              onClick={
                !isUpdating ? handleSGELoginRetry : handleResetSGEAccount
              }
              classes={{
                root: 'btn-highlight',
                label: 'text-16-bold',
              }}
              style={{ height: '40px' }}
            >
              <div>{!isUpdating ? 'Vérifier les infos' : 'Jy vais'}</div>
            </Button>
          </div>
        )
      default:
        //DEFAULT FOOTER BUTTONS
        //TODO change default button
        return (
          <Button
            aria-label={t('konnector_modal.accessibility.button_close')}
            onClick={() => handleCloseClick(state === SUCCESS_EVENT)}
            classes={{
              root: 'btn-highlight',
              label: 'text-16-bold',
            }}
            style={{ height: '40px' }}
          >
            <div>{t('konnector_modal.button_validate')}</div>
          </Button>
        )
    }
  }

  const successButton = () => {
    //DEFAULT FOOTER BUTTONS
    return (
      <Button
        aria-label={t('konnector_modal.accessibility.button_close')}
        onClick={() => handleCloseClick(state === SUCCESS_EVENT)}
        classes={{
          root: 'btn-highlight',
          label: 'text-16-bold',
        }}
      >
        <div>{t('konnector_modal.button_validate')}</div>
      </Button>
    )
  }

  return (
    <>
      {state === ERROR_EVENT && fluidType === FluidType.ELECTRICITY
        ? errorButtons()
        : successButton()}
    </>
  )
}

export default KonnectorModalFooter
