import React, { useCallback, useEffect, useState } from 'react'
import './partnerConnectionStepModal.scss'
import { Step } from 'models/step.model'
import { Button, Dialog, IconButton } from '@material-ui/core'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import CloseIcon from 'assets/icons/ico/close.svg'
import StepDetail from './StepDetail'
import { FluidType } from 'enum/fluid.enum'
import classNames from 'classnames'
import { getPartnerSteps } from 'utils/steps'

interface PartnerConnectionStepsModalProps {
  fluidType: FluidType
  open: boolean
  handleCloseClick: () => void
  handleEndSteps: () => void
}

const PartnerConnectionStepsModal = ({
  fluidType,
  open,
  handleCloseClick,
  handleEndSteps,
}: PartnerConnectionStepsModalProps) => {
  const { t } = useI18n()

  const [steps, setSteps] = useState<Step[]>([])
  const [stepIndex, setStepIndex] = useState<number>(0)

  const goPrevStep = useCallback(() => {
    setStepIndex(prev => prev - 1)
  }, [])

  const goNextStep = useCallback(() => {
    setStepIndex(prev => prev + 1)
  }, [])

  useEffect(() => {
    if (open) {
      const newSteps: Step[] = getPartnerSteps(fluidType)
      setSteps(newSteps)
      setStepIndex(0)
    }
  }, [open, fluidType])

  return (
    <Dialog
      open={open && steps.length > 0}
      onClose={handleCloseClick}
      aria-labelledby={'accessibility-title'}
      classes={{
        root: 'modal-root',
        paper: 'modal-paper',
      }}
    >
      <div id={'accessibility-title'}>
        {t('partner_connection_steps.accessibility.window_title')}
      </div>
      <IconButton
        aria-label={t('partner_connection_steps.accessibility.button_close')}
        className="modal-paper-close-button"
        onClick={handleCloseClick}
      >
        <Icon icon={CloseIcon} size={16} />
      </IconButton>
      <div className="partners-connection-step-content">
        <div className="partners-connection-step-progress">{`${
          stepIndex + 1
        } / ${steps.length}`}</div>
        <StepDetail step={steps[stepIndex]} />
      </div>
      <div className="partners-connection-step-navigation">
        <Button
          aria-label={t(
            'partner_connection_steps.accessibility.button_previous'
          )}
          onClick={goPrevStep}
          className={classNames('navigation-button', {
            ['disabled']: stepIndex === 0,
          })}
          disabled={stepIndex === 0}
          classes={{
            root: 'btn-profile-back',
            label: 'text-16-bold',
          }}
        >
          {`< ${t('partner_connection_steps.button_previous')}`}
        </Button>
        <Button
          aria-label={
            stepIndex === steps.length - 1
              ? t('partner_connection_steps.accessibility.button_validate')
              : t('partner_connection_steps.accessibility.button_next')
          }
          onClick={stepIndex === steps.length - 1 ? handleEndSteps : goNextStep}
          className={'navigation-button'}
          classes={{
            root: 'btn-profile-next rounded',
            label: 'text-16-bold',
          }}
        >
          {stepIndex === steps.length - 1
            ? t(
                `partner_connection_steps.${FluidType[
                  fluidType
                ].toLowerCase()}.button_validate`
              )
            : `${t('partner_connection_steps.button_next')} >`}
        </Button>
      </div>
    </Dialog>
  )
}

export default PartnerConnectionStepsModal
