import { Season } from 'enum/ecogesture.enum'
import { UserActionState } from 'enum/userAction.enum'
import { DateTime } from 'luxon'
import { Ecogesture, UserAction } from 'models'
import {
  AllEcogestureData,
  defaultEcogestureData,
  ecogestureDefault,
} from '../../tests/__mocks__/actionData.mock'
import mockClient from '../../tests/__mocks__/client'
import { userChallengeData } from '../../tests/__mocks__/userChallengeData.mock'
import ActionService from './action.service'
import * as utils from 'utils/utils'

const mockgetAllUserChallengeEntities = jest.fn()
jest.mock('./challenge.service', () => {
  return jest.fn(() => {
    return {
      getAllUserChallengeEntities: mockgetAllUserChallengeEntities,
    }
  })
})

const mockgetAllEcogestures = jest.fn()
const mockgetEcogesturesByIds = jest.fn()
jest.mock('./ecogesture.service', () => {
  return jest.fn(() => {
    return {
      getAllEcogestures: mockgetAllEcogestures,
      getEcogesturesByIds: mockgetEcogesturesByIds,
    }
  })
})

describe('Action Service', () => {
  const actionService = new ActionService(mockClient)

  it('should return the default actions', async () => {
    mockgetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
    mockgetEcogesturesByIds.mockResolvedValueOnce(defaultEcogestureData)

    const result: Ecogesture[] = await actionService.getDefaultActions()
    expect(result).toEqual(defaultEcogestureData)
  })
  it('should return the Available Action List', async () => {
    mockgetAllUserChallengeEntities.mockResolvedValueOnce(userChallengeData)
    mockgetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
    mockgetEcogesturesByIds.mockResolvedValueOnce([
      AllEcogestureData[2],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])

    const result: Ecogesture[] = await actionService.getAvailableActionList()
    expect(result).toEqual([
      AllEcogestureData[2],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])
  })
  it('should return winter ecogestures', () => {
    jest.spyOn(utils, 'getSeason').mockReturnValueOnce(Season.WINTER)
    const result: Ecogesture[] = actionService.filterBySeason(AllEcogestureData)
    expect(result).toEqual([
      AllEcogestureData[0],
      AllEcogestureData[2],
      AllEcogestureData[5],
    ])
  })
  it('should return no season ecogestures', () => {
    jest.spyOn(utils, 'getSeason').mockReturnValueOnce(Season.NONE)
    const result: Ecogesture[] = actionService.filterBySeason(AllEcogestureData)

    expect(result).toEqual([AllEcogestureData[3], AllEcogestureData[4]])
  })
  it('should return summer ecogestures', () => {
    jest.spyOn(utils, 'getSeason').mockReturnValueOnce(Season.SUMMER)
    const result: Ecogesture[] = actionService.filterBySeason(AllEcogestureData)

    expect(result).toEqual([
      AllEcogestureData[1],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])
  })

  it('should launch an action', () => {
    jest
      .spyOn(DateTime, 'local')
      .mockReturnValueOnce(
        DateTime.fromISO('2020-10-01T00:00:00.000Z', { zone: 'utc' })
      )
    const result: UserAction = actionService.launchAction(ecogestureDefault)
    const UserActionDefault: UserAction = {
      ecogesture: ecogestureDefault,
      startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
        zone: 'utc',
      }),
      state: UserActionState.ONGOING,
    }
    expect(result).toEqual(UserActionDefault)
  })
  it('complete list if < 3', async () => {
    mockgetEcogesturesByIds.mockResolvedValueOnce([
      AllEcogestureData[2],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])
    const result = await actionService.keepListComplete([
      AllEcogestureData[0],
      AllEcogestureData[1],
    ])

    expect(result).toEqual([
      AllEcogestureData[0],
      AllEcogestureData[1],
      AllEcogestureData[2],
    ])
  })

  describe('getCustomActions function', () => {
    it('should return filtered and sorted ecogestures and not complete the list', async () => {
      jest.spyOn(utils, 'getSeason').mockReturnValueOnce(Season.WINTER)
      mockgetAllUserChallengeEntities.mockResolvedValueOnce(userChallengeData)
      mockgetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
      mockgetEcogesturesByIds.mockResolvedValueOnce([
        AllEcogestureData[0],
        AllEcogestureData[1],
        AllEcogestureData[2],
        AllEcogestureData[3],
        AllEcogestureData[4],
        AllEcogestureData[5],
      ])

      const result = await actionService.getCustomActions([0, 2])
      expect(result).toEqual([
        AllEcogestureData[0],
        AllEcogestureData[5],
        AllEcogestureData[2],
      ])
    })
    it('should return filtered and sorted ecogestures and complete the list with default ecogesture', async () => {
      jest.spyOn(utils, 'getSeason').mockReturnValueOnce(Season.WINTER)
      mockgetAllUserChallengeEntities.mockResolvedValueOnce(userChallengeData)
      mockgetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
      mockgetEcogesturesByIds.mockResolvedValueOnce([
        AllEcogestureData[0],
        AllEcogestureData[1],
        AllEcogestureData[2],
        AllEcogestureData[3],
      ])
      mockgetEcogesturesByIds.mockClear()
      mockgetEcogesturesByIds.mockResolvedValueOnce([
        defaultEcogestureData[0],
        defaultEcogestureData[1],
        defaultEcogestureData[2],
      ])
      const result = await actionService.getCustomActions([0, 2])
      expect(result).toEqual([
        AllEcogestureData[0],
        defaultEcogestureData[0],
        AllEcogestureData[2],
      ])
    })
  })
})
